#pragma once
#include <LibHTTP/HeaderMap.h>
#include <LibURL/URL.h>
#include <AK/Error.h>
#include <AK/MemoryStream.h>
#include <AK/OwnPtr.h>
#include <AK/Result.h>
#include <AK/Utf8View.h>
#include <LibIPC/Connection.h>
#include <LibIPC/Decoder.h>
#include <LibIPC/Encoder.h>
#include <LibIPC/File.h>
#include <LibIPC/Message.h>
#include <LibIPC/Stub.h>

#if defined(AK_COMPILER_CLANG)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdefaulted-function-deleted"
#endif

namespace Messages::RequestClient {

enum class MessageID : i32 {
    RequestStarted = 1,
    RequestFinished = 2,
    HeadersBecameAvailable = 3,
    WebsocketConnected = 4,
    WebsocketReceived = 5,
    WebsocketErrored = 6,
    WebsocketClosed = 7,
    WebsocketReadyStateChanged = 8,
    WebsocketSubprotocol = 9,
    WebsocketCertificateRequested = 10,
    CertificateRequested = 11,
};

class RequestStarted final : public IPC::Message {
public:

    RequestStarted(decltype(nullptr)) : m_ipc_message_valid(false) { }
    RequestStarted(RequestStarted const&) = default;
    RequestStarted(RequestStarted&&) = default;
    RequestStarted& operator=(RequestStarted const&) = default;
    RequestStarted(i32 request_id, IPC::File fd) : m_request_id(move(request_id)), m_fd(move(fd)) {}

    virtual ~RequestStarted() override {}

    virtual u32 endpoint_magic() const override { return 1502652576; }
    virtual i32 message_id() const override { return (int)MessageID::RequestStarted; }
    static i32 static_message_id() { return (int)MessageID::RequestStarted; }
    virtual const char* message_name() const override { return "RequestClient::RequestStarted"; }

    static ErrorOr<NonnullOwnPtr<RequestStarted>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto request_id = TRY((decoder.decode<i32>()));

        auto fd = TRY((decoder.decode<IPC::File>()));

        return make<RequestStarted>(move(request_id), move(fd));
    }

    virtual bool valid() const override { return m_ipc_message_valid; }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        VERIFY(valid());

        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::RequestStarted));

        TRY(stream.encode(m_request_id));

        TRY(stream.encode(m_fd));

        return buffer;
    }

    const i32& request_id() const { return m_request_id; }
    i32 take_request_id() { return move(m_request_id); }

    const IPC::File& fd() const { return m_fd; }
    IPC::File take_fd() { return move(m_fd); }

private:
    bool m_ipc_message_valid { true };

    i32 m_request_id {};

    IPC::File m_fd {};

};

class RequestFinished final : public IPC::Message {
public:

    RequestFinished(decltype(nullptr)) : m_ipc_message_valid(false) { }
    RequestFinished(RequestFinished const&) = default;
    RequestFinished(RequestFinished&&) = default;
    RequestFinished& operator=(RequestFinished const&) = default;
    RequestFinished(i32 request_id, bool success, u64 total_size) : m_request_id(move(request_id)), m_success(move(success)), m_total_size(move(total_size)) {}

    virtual ~RequestFinished() override {}

    virtual u32 endpoint_magic() const override { return 1502652576; }
    virtual i32 message_id() const override { return (int)MessageID::RequestFinished; }
    static i32 static_message_id() { return (int)MessageID::RequestFinished; }
    virtual const char* message_name() const override { return "RequestClient::RequestFinished"; }

    static ErrorOr<NonnullOwnPtr<RequestFinished>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto request_id = TRY((decoder.decode<i32>()));

        auto success = TRY((decoder.decode<bool>()));

        auto total_size = TRY((decoder.decode<u64>()));

        return make<RequestFinished>(move(request_id), move(success), move(total_size));
    }

    virtual bool valid() const override { return m_ipc_message_valid; }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        VERIFY(valid());

        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::RequestFinished));

        TRY(stream.encode(m_request_id));

        TRY(stream.encode(m_success));

        TRY(stream.encode(m_total_size));

        return buffer;
    }

    const i32& request_id() const { return m_request_id; }
    i32 take_request_id() { return move(m_request_id); }

    const bool& success() const { return m_success; }
    bool take_success() { return move(m_success); }

    const u64& total_size() const { return m_total_size; }
    u64 take_total_size() { return move(m_total_size); }

private:
    bool m_ipc_message_valid { true };

    i32 m_request_id {};

    bool m_success {};

    u64 m_total_size {};

};

class HeadersBecameAvailable final : public IPC::Message {
public:

    HeadersBecameAvailable(decltype(nullptr)) : m_ipc_message_valid(false) { }
    HeadersBecameAvailable(HeadersBecameAvailable const&) = default;
    HeadersBecameAvailable(HeadersBecameAvailable&&) = default;
    HeadersBecameAvailable& operator=(HeadersBecameAvailable const&) = default;
    HeadersBecameAvailable(i32 request_id, HTTP::HeaderMap response_headers, Optional<u32> status_code) : m_request_id(move(request_id)), m_response_headers(move(response_headers)), m_status_code(move(status_code)) {}

    virtual ~HeadersBecameAvailable() override {}

    virtual u32 endpoint_magic() const override { return 1502652576; }
    virtual i32 message_id() const override { return (int)MessageID::HeadersBecameAvailable; }
    static i32 static_message_id() { return (int)MessageID::HeadersBecameAvailable; }
    virtual const char* message_name() const override { return "RequestClient::HeadersBecameAvailable"; }

    static ErrorOr<NonnullOwnPtr<HeadersBecameAvailable>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto request_id = TRY((decoder.decode<i32>()));

        auto response_headers = TRY((decoder.decode<HTTP::HeaderMap>()));

        auto status_code = TRY((decoder.decode<Optional<u32>>()));

        return make<HeadersBecameAvailable>(move(request_id), move(response_headers), move(status_code));
    }

    virtual bool valid() const override { return m_ipc_message_valid; }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        VERIFY(valid());

        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::HeadersBecameAvailable));

        TRY(stream.encode(m_request_id));

        TRY(stream.encode(m_response_headers));

        TRY(stream.encode(m_status_code));

        return buffer;
    }

    const i32& request_id() const { return m_request_id; }
    i32 take_request_id() { return move(m_request_id); }

    const HTTP::HeaderMap& response_headers() const { return m_response_headers; }
    HTTP::HeaderMap take_response_headers() { return move(m_response_headers); }

    const Optional<u32>& status_code() const { return m_status_code; }
    Optional<u32> take_status_code() { return move(m_status_code); }

private:
    bool m_ipc_message_valid { true };

    i32 m_request_id {};

    HTTP::HeaderMap m_response_headers {};

    Optional<u32> m_status_code {};

};

class WebsocketConnected final : public IPC::Message {
public:

    WebsocketConnected(decltype(nullptr)) : m_ipc_message_valid(false) { }
    WebsocketConnected(WebsocketConnected const&) = default;
    WebsocketConnected(WebsocketConnected&&) = default;
    WebsocketConnected& operator=(WebsocketConnected const&) = default;
    WebsocketConnected(i64 websocket_id) : m_websocket_id(move(websocket_id)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, i64>)
    WebsocketConnected(WrappedReturnType&& value)
        : m_websocket_id(forward<WrappedReturnType>(value))
    {
    }

    virtual ~WebsocketConnected() override {}

    virtual u32 endpoint_magic() const override { return 1502652576; }
    virtual i32 message_id() const override { return (int)MessageID::WebsocketConnected; }
    static i32 static_message_id() { return (int)MessageID::WebsocketConnected; }
    virtual const char* message_name() const override { return "RequestClient::WebsocketConnected"; }

    static ErrorOr<NonnullOwnPtr<WebsocketConnected>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto websocket_id = TRY((decoder.decode<i64>()));

        return make<WebsocketConnected>(move(websocket_id));
    }

    virtual bool valid() const override { return m_ipc_message_valid; }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        VERIFY(valid());

        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::WebsocketConnected));

        TRY(stream.encode(m_websocket_id));

        return buffer;
    }

    const i64& websocket_id() const { return m_websocket_id; }
    i64 take_websocket_id() { return move(m_websocket_id); }

private:
    bool m_ipc_message_valid { true };

    i64 m_websocket_id {};

};

class WebsocketReceived final : public IPC::Message {
public:

    WebsocketReceived(decltype(nullptr)) : m_ipc_message_valid(false) { }
    WebsocketReceived(WebsocketReceived const&) = default;
    WebsocketReceived(WebsocketReceived&&) = default;
    WebsocketReceived& operator=(WebsocketReceived const&) = default;
    WebsocketReceived(i64 websocket_id, bool is_text, ByteBuffer data) : m_websocket_id(move(websocket_id)), m_is_text(move(is_text)), m_data(move(data)) {}

    virtual ~WebsocketReceived() override {}

    virtual u32 endpoint_magic() const override { return 1502652576; }
    virtual i32 message_id() const override { return (int)MessageID::WebsocketReceived; }
    static i32 static_message_id() { return (int)MessageID::WebsocketReceived; }
    virtual const char* message_name() const override { return "RequestClient::WebsocketReceived"; }

    static ErrorOr<NonnullOwnPtr<WebsocketReceived>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto websocket_id = TRY((decoder.decode<i64>()));

        auto is_text = TRY((decoder.decode<bool>()));

        auto data = TRY((decoder.decode<ByteBuffer>()));

        return make<WebsocketReceived>(move(websocket_id), move(is_text), move(data));
    }

    virtual bool valid() const override { return m_ipc_message_valid; }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        VERIFY(valid());

        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::WebsocketReceived));

        TRY(stream.encode(m_websocket_id));

        TRY(stream.encode(m_is_text));

        TRY(stream.encode(m_data));

        return buffer;
    }

    const i64& websocket_id() const { return m_websocket_id; }
    i64 take_websocket_id() { return move(m_websocket_id); }

    const bool& is_text() const { return m_is_text; }
    bool take_is_text() { return move(m_is_text); }

    const ByteBuffer& data() const { return m_data; }
    ByteBuffer take_data() { return move(m_data); }

private:
    bool m_ipc_message_valid { true };

    i64 m_websocket_id {};

    bool m_is_text {};

    ByteBuffer m_data {};

};

class WebsocketErrored final : public IPC::Message {
public:

    WebsocketErrored(decltype(nullptr)) : m_ipc_message_valid(false) { }
    WebsocketErrored(WebsocketErrored const&) = default;
    WebsocketErrored(WebsocketErrored&&) = default;
    WebsocketErrored& operator=(WebsocketErrored const&) = default;
    WebsocketErrored(i64 websocket_id, i32 message) : m_websocket_id(move(websocket_id)), m_message(move(message)) {}

    virtual ~WebsocketErrored() override {}

    virtual u32 endpoint_magic() const override { return 1502652576; }
    virtual i32 message_id() const override { return (int)MessageID::WebsocketErrored; }
    static i32 static_message_id() { return (int)MessageID::WebsocketErrored; }
    virtual const char* message_name() const override { return "RequestClient::WebsocketErrored"; }

    static ErrorOr<NonnullOwnPtr<WebsocketErrored>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto websocket_id = TRY((decoder.decode<i64>()));

        auto message = TRY((decoder.decode<i32>()));

        return make<WebsocketErrored>(move(websocket_id), move(message));
    }

    virtual bool valid() const override { return m_ipc_message_valid; }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        VERIFY(valid());

        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::WebsocketErrored));

        TRY(stream.encode(m_websocket_id));

        TRY(stream.encode(m_message));

        return buffer;
    }

    const i64& websocket_id() const { return m_websocket_id; }
    i64 take_websocket_id() { return move(m_websocket_id); }

    const i32& message() const { return m_message; }
    i32 take_message() { return move(m_message); }

private:
    bool m_ipc_message_valid { true };

    i64 m_websocket_id {};

    i32 m_message {};

};

class WebsocketClosed final : public IPC::Message {
public:

    WebsocketClosed(decltype(nullptr)) : m_ipc_message_valid(false) { }
    WebsocketClosed(WebsocketClosed const&) = default;
    WebsocketClosed(WebsocketClosed&&) = default;
    WebsocketClosed& operator=(WebsocketClosed const&) = default;
    WebsocketClosed(i64 websocket_id, u16 code, ByteString reason, bool clean) : m_websocket_id(move(websocket_id)), m_code(move(code)), m_reason(move(reason)), m_clean(move(clean)) {}

    virtual ~WebsocketClosed() override {}

    virtual u32 endpoint_magic() const override { return 1502652576; }
    virtual i32 message_id() const override { return (int)MessageID::WebsocketClosed; }
    static i32 static_message_id() { return (int)MessageID::WebsocketClosed; }
    virtual const char* message_name() const override { return "RequestClient::WebsocketClosed"; }

    static ErrorOr<NonnullOwnPtr<WebsocketClosed>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto websocket_id = TRY((decoder.decode<i64>()));

        auto code = TRY((decoder.decode<u16>()));

        auto reason = TRY((decoder.decode<ByteString>()));

        auto clean = TRY((decoder.decode<bool>()));

        return make<WebsocketClosed>(move(websocket_id), move(code), move(reason), move(clean));
    }

    virtual bool valid() const override { return m_ipc_message_valid; }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        VERIFY(valid());

        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::WebsocketClosed));

        TRY(stream.encode(m_websocket_id));

        TRY(stream.encode(m_code));

        TRY(stream.encode(m_reason));

        TRY(stream.encode(m_clean));

        return buffer;
    }

    const i64& websocket_id() const { return m_websocket_id; }
    i64 take_websocket_id() { return move(m_websocket_id); }

    const u16& code() const { return m_code; }
    u16 take_code() { return move(m_code); }

    const ByteString& reason() const { return m_reason; }
    ByteString take_reason() { return move(m_reason); }

    const bool& clean() const { return m_clean; }
    bool take_clean() { return move(m_clean); }

private:
    bool m_ipc_message_valid { true };

    i64 m_websocket_id {};

    u16 m_code {};

    ByteString m_reason {};

    bool m_clean {};

};

class WebsocketReadyStateChanged final : public IPC::Message {
public:

    WebsocketReadyStateChanged(decltype(nullptr)) : m_ipc_message_valid(false) { }
    WebsocketReadyStateChanged(WebsocketReadyStateChanged const&) = default;
    WebsocketReadyStateChanged(WebsocketReadyStateChanged&&) = default;
    WebsocketReadyStateChanged& operator=(WebsocketReadyStateChanged const&) = default;
    WebsocketReadyStateChanged(i64 websocket_id, u32 ready_state) : m_websocket_id(move(websocket_id)), m_ready_state(move(ready_state)) {}

    virtual ~WebsocketReadyStateChanged() override {}

    virtual u32 endpoint_magic() const override { return 1502652576; }
    virtual i32 message_id() const override { return (int)MessageID::WebsocketReadyStateChanged; }
    static i32 static_message_id() { return (int)MessageID::WebsocketReadyStateChanged; }
    virtual const char* message_name() const override { return "RequestClient::WebsocketReadyStateChanged"; }

    static ErrorOr<NonnullOwnPtr<WebsocketReadyStateChanged>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto websocket_id = TRY((decoder.decode<i64>()));

        auto ready_state = TRY((decoder.decode<u32>()));

        return make<WebsocketReadyStateChanged>(move(websocket_id), move(ready_state));
    }

    virtual bool valid() const override { return m_ipc_message_valid; }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        VERIFY(valid());

        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::WebsocketReadyStateChanged));

        TRY(stream.encode(m_websocket_id));

        TRY(stream.encode(m_ready_state));

        return buffer;
    }

    const i64& websocket_id() const { return m_websocket_id; }
    i64 take_websocket_id() { return move(m_websocket_id); }

    const u32& ready_state() const { return m_ready_state; }
    u32 take_ready_state() { return move(m_ready_state); }

private:
    bool m_ipc_message_valid { true };

    i64 m_websocket_id {};

    u32 m_ready_state {};

};

class WebsocketSubprotocol final : public IPC::Message {
public:

    WebsocketSubprotocol(decltype(nullptr)) : m_ipc_message_valid(false) { }
    WebsocketSubprotocol(WebsocketSubprotocol const&) = default;
    WebsocketSubprotocol(WebsocketSubprotocol&&) = default;
    WebsocketSubprotocol& operator=(WebsocketSubprotocol const&) = default;
    WebsocketSubprotocol(i64 websocket_id, ByteString subprotocol) : m_websocket_id(move(websocket_id)), m_subprotocol(move(subprotocol)) {}

    virtual ~WebsocketSubprotocol() override {}

    virtual u32 endpoint_magic() const override { return 1502652576; }
    virtual i32 message_id() const override { return (int)MessageID::WebsocketSubprotocol; }
    static i32 static_message_id() { return (int)MessageID::WebsocketSubprotocol; }
    virtual const char* message_name() const override { return "RequestClient::WebsocketSubprotocol"; }

    static ErrorOr<NonnullOwnPtr<WebsocketSubprotocol>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto websocket_id = TRY((decoder.decode<i64>()));

        auto subprotocol = TRY((decoder.decode<ByteString>()));

        return make<WebsocketSubprotocol>(move(websocket_id), move(subprotocol));
    }

    virtual bool valid() const override { return m_ipc_message_valid; }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        VERIFY(valid());

        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::WebsocketSubprotocol));

        TRY(stream.encode(m_websocket_id));

        TRY(stream.encode(m_subprotocol));

        return buffer;
    }

    const i64& websocket_id() const { return m_websocket_id; }
    i64 take_websocket_id() { return move(m_websocket_id); }

    const ByteString& subprotocol() const { return m_subprotocol; }
    ByteString take_subprotocol() { return move(m_subprotocol); }

private:
    bool m_ipc_message_valid { true };

    i64 m_websocket_id {};

    ByteString m_subprotocol {};

};

class WebsocketCertificateRequested final : public IPC::Message {
public:

    WebsocketCertificateRequested(decltype(nullptr)) : m_ipc_message_valid(false) { }
    WebsocketCertificateRequested(WebsocketCertificateRequested const&) = default;
    WebsocketCertificateRequested(WebsocketCertificateRequested&&) = default;
    WebsocketCertificateRequested& operator=(WebsocketCertificateRequested const&) = default;
    WebsocketCertificateRequested(i64 websocket_id) : m_websocket_id(move(websocket_id)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, i64>)
    WebsocketCertificateRequested(WrappedReturnType&& value)
        : m_websocket_id(forward<WrappedReturnType>(value))
    {
    }

    virtual ~WebsocketCertificateRequested() override {}

    virtual u32 endpoint_magic() const override { return 1502652576; }
    virtual i32 message_id() const override { return (int)MessageID::WebsocketCertificateRequested; }
    static i32 static_message_id() { return (int)MessageID::WebsocketCertificateRequested; }
    virtual const char* message_name() const override { return "RequestClient::WebsocketCertificateRequested"; }

    static ErrorOr<NonnullOwnPtr<WebsocketCertificateRequested>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto websocket_id = TRY((decoder.decode<i64>()));

        return make<WebsocketCertificateRequested>(move(websocket_id));
    }

    virtual bool valid() const override { return m_ipc_message_valid; }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        VERIFY(valid());

        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::WebsocketCertificateRequested));

        TRY(stream.encode(m_websocket_id));

        return buffer;
    }

    const i64& websocket_id() const { return m_websocket_id; }
    i64 take_websocket_id() { return move(m_websocket_id); }

private:
    bool m_ipc_message_valid { true };

    i64 m_websocket_id {};

};

class CertificateRequested final : public IPC::Message {
public:

    CertificateRequested(decltype(nullptr)) : m_ipc_message_valid(false) { }
    CertificateRequested(CertificateRequested const&) = default;
    CertificateRequested(CertificateRequested&&) = default;
    CertificateRequested& operator=(CertificateRequested const&) = default;
    CertificateRequested(i32 request_id) : m_request_id(move(request_id)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, i32>)
    CertificateRequested(WrappedReturnType&& value)
        : m_request_id(forward<WrappedReturnType>(value))
    {
    }

    virtual ~CertificateRequested() override {}

    virtual u32 endpoint_magic() const override { return 1502652576; }
    virtual i32 message_id() const override { return (int)MessageID::CertificateRequested; }
    static i32 static_message_id() { return (int)MessageID::CertificateRequested; }
    virtual const char* message_name() const override { return "RequestClient::CertificateRequested"; }

    static ErrorOr<NonnullOwnPtr<CertificateRequested>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto request_id = TRY((decoder.decode<i32>()));

        return make<CertificateRequested>(move(request_id));
    }

    virtual bool valid() const override { return m_ipc_message_valid; }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        VERIFY(valid());

        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::CertificateRequested));

        TRY(stream.encode(m_request_id));

        return buffer;
    }

    const i32& request_id() const { return m_request_id; }
    i32 take_request_id() { return move(m_request_id); }

private:
    bool m_ipc_message_valid { true };

    i32 m_request_id {};

};

} // namespace Messages::RequestClient

template<typename LocalEndpoint, typename PeerEndpoint>
class RequestClientProxy {
public:
    // Used to disambiguate the constructor call.
    struct Tag { };

    RequestClientProxy(IPC::Connection<LocalEndpoint, PeerEndpoint>& connection, Tag)
        : m_connection(connection)
    { }

    void async_request_started(
i32 request_id, IPC::File fd) {
        // FIXME: Handle post_message failures.
        (void) m_connection.post_message(Messages::RequestClient::RequestStarted { request_id, move(fd) });

    }

    void async_request_finished(
i32 request_id, bool success, u64 total_size) {
        // FIXME: Handle post_message failures.
        (void) m_connection.post_message(Messages::RequestClient::RequestFinished { request_id, success, total_size });

    }

    void async_headers_became_available(
i32 request_id, HTTP::HeaderMap response_headers, Optional<u32> status_code) {
        // FIXME: Handle post_message failures.
        (void) m_connection.post_message(Messages::RequestClient::HeadersBecameAvailable { request_id, move(response_headers), move(status_code) });

    }

    void async_websocket_connected(
i64 websocket_id) {
        // FIXME: Handle post_message failures.
        (void) m_connection.post_message(Messages::RequestClient::WebsocketConnected { websocket_id });

    }

    void async_websocket_received(
i64 websocket_id, bool is_text, ByteBuffer data) {
        // FIXME: Handle post_message failures.
        (void) m_connection.post_message(Messages::RequestClient::WebsocketReceived { websocket_id, is_text, move(data) });

    }

    void async_websocket_errored(
i64 websocket_id, i32 message) {
        // FIXME: Handle post_message failures.
        (void) m_connection.post_message(Messages::RequestClient::WebsocketErrored { websocket_id, message });

    }

    void async_websocket_closed(
i64 websocket_id, u16 code, ByteString reason, bool clean) {
        // FIXME: Handle post_message failures.
        (void) m_connection.post_message(Messages::RequestClient::WebsocketClosed { websocket_id, code, move(reason), clean });

    }

    void async_websocket_ready_state_changed(
i64 websocket_id, u32 ready_state) {
        // FIXME: Handle post_message failures.
        (void) m_connection.post_message(Messages::RequestClient::WebsocketReadyStateChanged { websocket_id, ready_state });

    }

    void async_websocket_subprotocol(
i64 websocket_id, ByteString subprotocol) {
        // FIXME: Handle post_message failures.
        (void) m_connection.post_message(Messages::RequestClient::WebsocketSubprotocol { websocket_id, move(subprotocol) });

    }

    void async_websocket_certificate_requested(
i64 websocket_id) {
        // FIXME: Handle post_message failures.
        (void) m_connection.post_message(Messages::RequestClient::WebsocketCertificateRequested { websocket_id });

    }

    void async_certificate_requested(
i32 request_id) {
        // FIXME: Handle post_message failures.
        (void) m_connection.post_message(Messages::RequestClient::CertificateRequested { request_id });

    }

private:
    IPC::Connection<LocalEndpoint, PeerEndpoint>& m_connection;
};

template<typename LocalEndpoint, typename PeerEndpoint>
class RequestClientProxy;
class RequestClientStub;

class RequestClientEndpoint {
public:
    template<typename LocalEndpoint>
    using Proxy = RequestClientProxy<LocalEndpoint, RequestClientEndpoint>;
    using Stub = RequestClientStub;

    static u32 static_magic() { return 1502652576; }

    static ErrorOr<NonnullOwnPtr<IPC::Message>> decode_message(ReadonlyBytes buffer, [[maybe_unused]] Queue<IPC::File>& files)
    {
        FixedMemoryStream stream { buffer };
        auto message_endpoint_magic = TRY(stream.read_value<u32>());

        if (message_endpoint_magic != 1502652576) {
            return Error::from_string_literal("Endpoint magic number mismatch, not my message!");
        }

        auto message_id = TRY(stream.read_value<i32>());


        switch (message_id) {

        case (int)Messages::RequestClient::MessageID::RequestStarted:
            return TRY(Messages::RequestClient::RequestStarted::decode(stream, files));
        case (int)Messages::RequestClient::MessageID::RequestFinished:
            return TRY(Messages::RequestClient::RequestFinished::decode(stream, files));
        case (int)Messages::RequestClient::MessageID::HeadersBecameAvailable:
            return TRY(Messages::RequestClient::HeadersBecameAvailable::decode(stream, files));
        case (int)Messages::RequestClient::MessageID::WebsocketConnected:
            return TRY(Messages::RequestClient::WebsocketConnected::decode(stream, files));
        case (int)Messages::RequestClient::MessageID::WebsocketReceived:
            return TRY(Messages::RequestClient::WebsocketReceived::decode(stream, files));
        case (int)Messages::RequestClient::MessageID::WebsocketErrored:
            return TRY(Messages::RequestClient::WebsocketErrored::decode(stream, files));
        case (int)Messages::RequestClient::MessageID::WebsocketClosed:
            return TRY(Messages::RequestClient::WebsocketClosed::decode(stream, files));
        case (int)Messages::RequestClient::MessageID::WebsocketReadyStateChanged:
            return TRY(Messages::RequestClient::WebsocketReadyStateChanged::decode(stream, files));
        case (int)Messages::RequestClient::MessageID::WebsocketSubprotocol:
            return TRY(Messages::RequestClient::WebsocketSubprotocol::decode(stream, files));
        case (int)Messages::RequestClient::MessageID::WebsocketCertificateRequested:
            return TRY(Messages::RequestClient::WebsocketCertificateRequested::decode(stream, files));
        case (int)Messages::RequestClient::MessageID::CertificateRequested:
            return TRY(Messages::RequestClient::CertificateRequested::decode(stream, files));
        default:
            return Error::from_string_literal("Failed to decode RequestClient message");
        }

        VERIFY_NOT_REACHED();
    }

};

class RequestClientStub : public IPC::Stub {
public:
    RequestClientStub() { }
    virtual ~RequestClientStub() override { }

    virtual u32 magic() const override { return 1502652576; }
    virtual ByteString name() const override { return "RequestClient"; }

    virtual ErrorOr<OwnPtr<IPC::MessageBuffer>> handle(const IPC::Message& message) override
    {
        switch (message.message_id()) {

        case (int)Messages::RequestClient::MessageID::RequestStarted: {

            [[maybe_unused]] auto& request = static_cast<const Messages::RequestClient::RequestStarted&>(message);
            request_started(request.request_id(), request.fd());
            return nullptr;

        }

        case (int)Messages::RequestClient::MessageID::RequestFinished: {

            [[maybe_unused]] auto& request = static_cast<const Messages::RequestClient::RequestFinished&>(message);
            request_finished(request.request_id(), request.success(), request.total_size());
            return nullptr;

        }

        case (int)Messages::RequestClient::MessageID::HeadersBecameAvailable: {

            [[maybe_unused]] auto& request = static_cast<const Messages::RequestClient::HeadersBecameAvailable&>(message);
            headers_became_available(request.request_id(), request.response_headers(), request.status_code());
            return nullptr;

        }

        case (int)Messages::RequestClient::MessageID::WebsocketConnected: {

            [[maybe_unused]] auto& request = static_cast<const Messages::RequestClient::WebsocketConnected&>(message);
            websocket_connected(request.websocket_id());
            return nullptr;

        }

        case (int)Messages::RequestClient::MessageID::WebsocketReceived: {

            [[maybe_unused]] auto& request = static_cast<const Messages::RequestClient::WebsocketReceived&>(message);
            websocket_received(request.websocket_id(), request.is_text(), request.data());
            return nullptr;

        }

        case (int)Messages::RequestClient::MessageID::WebsocketErrored: {

            [[maybe_unused]] auto& request = static_cast<const Messages::RequestClient::WebsocketErrored&>(message);
            websocket_errored(request.websocket_id(), request.message());
            return nullptr;

        }

        case (int)Messages::RequestClient::MessageID::WebsocketClosed: {

            [[maybe_unused]] auto& request = static_cast<const Messages::RequestClient::WebsocketClosed&>(message);
            websocket_closed(request.websocket_id(), request.code(), request.reason(), request.clean());
            return nullptr;

        }

        case (int)Messages::RequestClient::MessageID::WebsocketReadyStateChanged: {

            [[maybe_unused]] auto& request = static_cast<const Messages::RequestClient::WebsocketReadyStateChanged&>(message);
            websocket_ready_state_changed(request.websocket_id(), request.ready_state());
            return nullptr;

        }

        case (int)Messages::RequestClient::MessageID::WebsocketSubprotocol: {

            [[maybe_unused]] auto& request = static_cast<const Messages::RequestClient::WebsocketSubprotocol&>(message);
            websocket_subprotocol(request.websocket_id(), request.subprotocol());
            return nullptr;

        }

        case (int)Messages::RequestClient::MessageID::WebsocketCertificateRequested: {

            [[maybe_unused]] auto& request = static_cast<const Messages::RequestClient::WebsocketCertificateRequested&>(message);
            websocket_certificate_requested(request.websocket_id());
            return nullptr;

        }

        case (int)Messages::RequestClient::MessageID::CertificateRequested: {

            [[maybe_unused]] auto& request = static_cast<const Messages::RequestClient::CertificateRequested&>(message);
            certificate_requested(request.request_id());
            return nullptr;

        }

        default:
            return Error::from_string_literal("Unknown message ID for RequestClient endpoint");
        }
    }

    virtual void request_started(
[[maybe_unused]] i32 request_id, [[maybe_unused]] IPC::File const& fd) = 0;

    virtual void request_finished(
[[maybe_unused]] i32 request_id, [[maybe_unused]] bool success, [[maybe_unused]] u64 total_size) = 0;

    virtual void headers_became_available(
[[maybe_unused]] i32 request_id, [[maybe_unused]] HTTP::HeaderMap const& response_headers, [[maybe_unused]] Optional<u32> const& status_code) = 0;

    virtual void websocket_connected(
[[maybe_unused]] i64 websocket_id) = 0;

    virtual void websocket_received(
[[maybe_unused]] i64 websocket_id, [[maybe_unused]] bool is_text, [[maybe_unused]] ByteBuffer const& data) = 0;

    virtual void websocket_errored(
[[maybe_unused]] i64 websocket_id, [[maybe_unused]] i32 message) = 0;

    virtual void websocket_closed(
[[maybe_unused]] i64 websocket_id, [[maybe_unused]] u16 code, [[maybe_unused]] ByteString const& reason, [[maybe_unused]] bool clean) = 0;

    virtual void websocket_ready_state_changed(
[[maybe_unused]] i64 websocket_id, [[maybe_unused]] u32 ready_state) = 0;

    virtual void websocket_subprotocol(
[[maybe_unused]] i64 websocket_id, [[maybe_unused]] ByteString const& subprotocol) = 0;

    virtual void websocket_certificate_requested(
[[maybe_unused]] i64 websocket_id) = 0;

    virtual void certificate_requested(
[[maybe_unused]] i32 request_id) = 0;

private:
};

#if defined(AK_COMPILER_CLANG)
#pragma clang diagnostic pop
#endif
