#pragma once
#include <LibWeb/WebDriver/Response.h>
#include <AK/Error.h>
#include <AK/MemoryStream.h>
#include <AK/OwnPtr.h>
#include <AK/Result.h>
#include <AK/Utf8View.h>
#include <LibIPC/Connection.h>
#include <LibIPC/Decoder.h>
#include <LibIPC/Encoder.h>
#include <LibIPC/File.h>
#include <LibIPC/Message.h>
#include <LibIPC/Stub.h>

#if defined(AK_COMPILER_CLANG)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdefaulted-function-deleted"
#endif

namespace Messages::WebDriverServer {

enum class MessageID : i32 {
    ScriptExecuted = 1,
};

class ScriptExecuted final : public IPC::Message {
public:

    ScriptExecuted(decltype(nullptr)) : m_ipc_message_valid(false) { }
    ScriptExecuted(ScriptExecuted const&) = default;
    ScriptExecuted(ScriptExecuted&&) = default;
    ScriptExecuted& operator=(ScriptExecuted const&) = default;
    ScriptExecuted(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    ScriptExecuted(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~ScriptExecuted() override {}

    virtual u32 endpoint_magic() const override { return 2244075654; }
    virtual i32 message_id() const override { return (int)MessageID::ScriptExecuted; }
    static i32 static_message_id() { return (int)MessageID::ScriptExecuted; }
    virtual const char* message_name() const override { return "WebDriverServer::ScriptExecuted"; }

    static ErrorOr<NonnullOwnPtr<ScriptExecuted>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<ScriptExecuted>(move(response));
    }

    virtual bool valid() const override { return m_ipc_message_valid; }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        VERIFY(valid());

        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ScriptExecuted));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:
    bool m_ipc_message_valid { true };

    Web::WebDriver::Response m_response {};

};

} // namespace Messages::WebDriverServer

template<typename LocalEndpoint, typename PeerEndpoint>
class WebDriverServerProxy {
public:
    // Used to disambiguate the constructor call.
    struct Tag { };

    WebDriverServerProxy(IPC::Connection<LocalEndpoint, PeerEndpoint>& connection, Tag)
        : m_connection(connection)
    { }

    void async_script_executed(
Web::WebDriver::Response response) {
        // FIXME: Handle post_message failures.
        (void) m_connection.post_message(Messages::WebDriverServer::ScriptExecuted { move(response) });

    }

private:
    IPC::Connection<LocalEndpoint, PeerEndpoint>& m_connection;
};

template<typename LocalEndpoint, typename PeerEndpoint>
class WebDriverServerProxy;
class WebDriverServerStub;

class WebDriverServerEndpoint {
public:
    template<typename LocalEndpoint>
    using Proxy = WebDriverServerProxy<LocalEndpoint, WebDriverServerEndpoint>;
    using Stub = WebDriverServerStub;

    static u32 static_magic() { return 2244075654; }

    static ErrorOr<NonnullOwnPtr<IPC::Message>> decode_message(ReadonlyBytes buffer, [[maybe_unused]] Queue<IPC::File>& files)
    {
        FixedMemoryStream stream { buffer };
        auto message_endpoint_magic = TRY(stream.read_value<u32>());

        if (message_endpoint_magic != 2244075654) {
            return Error::from_string_literal("Endpoint magic number mismatch, not my message!");
        }

        auto message_id = TRY(stream.read_value<i32>());


        switch (message_id) {

        case (int)Messages::WebDriverServer::MessageID::ScriptExecuted:
            return TRY(Messages::WebDriverServer::ScriptExecuted::decode(stream, files));
        default:
            return Error::from_string_literal("Failed to decode WebDriverServer message");
        }

        VERIFY_NOT_REACHED();
    }

};

class WebDriverServerStub : public IPC::Stub {
public:
    WebDriverServerStub() { }
    virtual ~WebDriverServerStub() override { }

    virtual u32 magic() const override { return 2244075654; }
    virtual ByteString name() const override { return "WebDriverServer"; }

    virtual ErrorOr<OwnPtr<IPC::MessageBuffer>> handle(const IPC::Message& message) override
    {
        switch (message.message_id()) {

        case (int)Messages::WebDriverServer::MessageID::ScriptExecuted: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverServer::ScriptExecuted&>(message);
            script_executed(request.response());
            return nullptr;

        }

        default:
            return Error::from_string_literal("Unknown message ID for WebDriverServer endpoint");
        }
    }

    virtual void script_executed(
[[maybe_unused]] Web::WebDriver::Response const& response) = 0;

private:
};

#if defined(AK_COMPILER_CLANG)
#pragma clang diagnostic pop
#endif
